import React, { useContext, useState } from "react";
import { ControlContext } from "../WControlProvider/ControlContext";
import { FormGroup } from "../../exports";
export const WFormContext = React.createContext({
  form: new FormGroup({}),
  formLoadedData: null,
  formLoadParams: null,
  formLoadStatus: "",
  formLoadedError: null,
  formSavedData: null,
  formSaveParams: null,
  formSaveStatus: "",
  formSavedError: null,
  submit: () => {},
  submitted: false,
  markAsSubmitted: () => {}
});
export const WForm = props => {
  const {
    formConfig,
    renderOnError,
    children,
    globalErrorMessages
  } = props;
  const {
    submit,
    formLoadStatus,
    form,
    formLoadedError,
    formLoadedData,
    formLoadParams,
    formSaveStatus,
    formSavedError,
    formSavedData,
    formSaveParams
  } = formConfig;
  const [submitted, setSubmitted] = useState(false);
  const formContextProvider = {
    form,
    formLoadedData,
    formLoadParams,
    formLoadStatus,
    formLoadedError,
    formSavedData,
    formSaveParams,
    formSaveStatus,
    formSavedError,
    submitted,
    markAsSubmitted: () => {
      setSubmitted(true);
    },
    globalErrorMessages,
    submit
  };
  const controlContext = {
    status: formLoadStatus,
    control: form,
    errors: formLoadedError
  };
  return /*#__PURE__*/React.createElement(React.Fragment, null, formLoadStatus && formLoadStatus !== "FAILURE" && /*#__PURE__*/React.createElement("form", {
    onSubmit: e => {
      e.preventDefault();
      setSubmitted(true);
      formConfig.submit();
    }
  }, /*#__PURE__*/React.createElement(ControlContext.Provider, {
    value: controlContext
  }, /*#__PURE__*/React.createElement(WFormContext.Provider, {
    value: formContextProvider
  }, typeof children === "function" ? children(formContextProvider) : children))), formLoadStatus === "FAILURE" && /*#__PURE__*/React.createElement(React.Fragment, null, renderOnError && renderOnError(null)));
};
export const useFormContext = () => {
  const formContext = useContext(WFormContext);
  return formContext;
};