import { useReducer, useRef } from "react";
import { AsyncProcessor } from "../utils/LoadControl";
export class FormConfig {
  get formLoadedData() {
    return this._loadDataAsyncProcesor.data;
  }

  get formLoadParams() {
    return this._loadDataAsyncProcesor.params;
  }

  get formLoadStatus() {
    return this._loadDataAsyncProcesor.status;
  }

  get formLoadedError() {
    return this._loadDataAsyncProcesor.error;
  }

  loadProcessing(params) {
    this._loadDataAsyncProcesor.processing(params);
  }

  loadSucceed(data) {
    this._loadDataAsyncProcesor.succeed(data);

    const {
      createForm,
      onFormLoaded
    } = this.config;
    this.form = createForm(data, this.form);
    this.form.forceUpdate = this.forceUpdate;
    this.forceUpdate();

    if (onFormLoaded) {
      //TODO: remove setTimeout. The purpose of setTimeout is make sure that <WForm> has finish of render.
      // so <WFormGroup|WFormArray|WFormContro   subscribe={()=>{}}> subscribe method will called the first time.
      setTimeout(() => {
        onFormLoaded(this.form, data);
      }, 100);
    }
  }

  loadFailed(error) {
    this._loadDataAsyncProcesor.failed(error);
  }

  loadRetry(callback, params) {
    this._loadDataAsyncProcesor.retry(callback, params);
  }

  get formSavedData() {
    return this._saveDataAsyncProcesor.data;
  }

  get formSaveParams() {
    return this._saveDataAsyncProcesor.params;
  }

  get formSaveStatus() {
    return this._saveDataAsyncProcesor.status;
  }

  get formSavedError() {
    return this._saveDataAsyncProcesor.error;
  }

  formSaveProcessing(params) {
    this._saveDataAsyncProcesor.processing(params);
  }

  formSaveSucceed(data) {
    this._saveDataAsyncProcesor.succeed(data);
  }

  formSaveFailed(error) {
    this._saveDataAsyncProcesor.failed(error);
  }

  formSaveRetry(callback, params) {
    this._saveDataAsyncProcesor.retry(callback, params);
  }

  constructor(config, forceUpdate) {
    this.config = config;
    this.forceUpdate = forceUpdate;
    this.form = void 0;
    this._loadDataAsyncProcesor = void 0;
    this._saveDataAsyncProcesor = void 0;
    this._loadDataAsyncProcesor = new AsyncProcessor(forceUpdate);
    this._saveDataAsyncProcesor = new AsyncProcessor(forceUpdate);
    this.loadProcessing = this.loadProcessing.bind(this);
    this.loadSucceed = this.loadSucceed.bind(this);
    this.loadFailed = this.loadFailed.bind(this);
    this.loadRetry = this.loadRetry.bind(this);
  }

  static create(config, forceUpdate) {
    return new FormConfig(config, forceUpdate);
  }

  submit() {
    const {
      onSubmit
    } = this.config;
    onSubmit && onSubmit(this.form, this._saveDataAsyncProcesor);
  }

}
export const useFormConfig = props => {
  const [, forceUpdate] = useReducer(x => x + 1, 0);
  return useRef(FormConfig.create(props, forceUpdate)).current;
};