"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.useCurrentControl = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = require("react");

var _operators = require("rxjs/operators");

var _exports = require("../../exports");

var _ControlContext = require("./ControlContext");

var _WForm = require("../WForm/WForm");

const useCurrentControl = props => {
  const {
    inputControl,
    name,
    expectedControl,
    onChange,
    errorMessages
  } = props;
  const {
    control,
    status,
    errors
  } = (0, _ControlContext.useControlContext)();
  const [loadControl, setLoadControl] = (0, _react.useState)({
    status: "",
    control: new _exports.FormControl(),
    errors: null
  });
  (0, _react.useEffect)(() => {
    if (status === "SUCCESS") return;
    setLoadControl({
      control,
      status,
      errors
    });
  }, [status, control, errors]);
  const formContext = (0, _WForm.useFormContext)();
  const {
    submitted,
    globalErrorMessages,
    form
  } = formContext;
  const prevValueRef = (0, _react.useRef)();
  (0, _react.useEffect)(() => {
    let currentControl = null;
    if (status !== "SUCCESS") return;

    try {
      if (!inputControl) {
        if (name) {
          if (!control) return;

          if (!(control instanceof _exports.FormGroup)) {
            const messageError = {
              code: `ParentControl is not instance of ${expectedControl} name:${name}`
            };
            throw messageError;
          }

          const childControl = control.get(name);

          if (!childControl) {
            const messageError = {
              code: `Control with name :${name} was not found.`
            };
            throw messageError;
          }

          currentControl = childControl;
        } else {
          currentControl = control;
        }
      } else {
        currentControl = control;
      }

      if (currentControl.constructor.name !== expectedControl) {
        const messageError = {
          code: `Control with name :${name} is not instance of ${expectedControl}.`
        };
        throw messageError;
      }

      setLoadControl({
        status: "SUCCESS",
        control: currentControl,
        errors: null
      });
    } catch (errors) {
      setLoadControl(loadControl => (0, _extends2.default)({}, loadControl, {
        status: "FAILURE",
        errors
      }));
    }
  }, [name, inputControl, status, control, expectedControl]);
  const [internalValidation, setInternalValidation] = (0, _react.useState)({
    errors: null
  });
  const [validation, setValidation] = (0, _react.useState)({
    errors: null
  });
  (0, _react.useEffect)(() => {
    if (loadControl.status !== "SUCCESS") return;
    const subs = loadControl.control.statusChanges.pipe((0, _operators.startWith)(loadControl.control.status), (0, _operators.map)(status => {
      return {
        status,
        errors: loadControl.control.errors
      };
    }), (0, _operators.tap)(validation => {
      if (validation.status !== "INVALID") {
        setValidation(validation);
      }

      setInternalValidation(validation);
    })).subscribe();
    return () => {
      subs.unsubscribe();
    };
  }, [loadControl.control, loadControl.status]);
  const [displayErrors, setDisplayErrors] = (0, _react.useState)(false);
  const touched = loadControl.control && loadControl.control.touched;
  const dirty = loadControl.control && loadControl.control.dirty;
  (0, _react.useEffect)(() => {
    const displayErrors = touched && dirty || submitted;
    setDisplayErrors(displayErrors);
  }, [touched, dirty, submitted]);
  const subscriberRef = (0, _react.useRef)();
  (0, _react.useEffect)(() => {
    if (loadControl.status !== "SUCCESS") return;
    if (subscriberRef.current) return;

    if (onChange) {
      subscriberRef.current = onChange;
    }
  }, [onChange, loadControl.status]);
  (0, _react.useEffect)(() => {
    if (loadControl.status !== "SUCCESS") return;
    const subscriber = subscriberRef.current;
    if (!subscriber) return;
    loadControl.control.valueChanges.subscribe(value => {
      subscriber({
        value,
        prevValue: prevValueRef.current,
        control: loadControl.control,
        parentControl: control,
        rootControl: form
      });
      prevValueRef.current = value;
    });
  }, [control, form, loadControl.status, loadControl.control]);
  (0, _react.useEffect)(() => {
    const {
      errors,
      status
    } = internalValidation;
    if (status !== "INVALID") return;
    if (!displayErrors) return;
    const errorsCalculated = Object.keys(errors || {}).reduce((t, e) => {
      const value = (errorMessages || {})[e] || (globalErrorMessages || {})[e] || e;
      const errorValue = typeof value === "function" ? value(errors[e]) : value;
      t[e] = errorValue;
      return t;
    }, {});
    setValidation({
      errors: errorsCalculated,
      status
    });
  }, [internalValidation, globalErrorMessages, errorMessages, displayErrors]);
  const controlContext = {
    status: loadControl.status,
    control: loadControl.control || new _exports.FormControl(),
    errors: loadControl.errors,
    validationStatus: validation.status,
    validationErrors: validation.errors
  };
  return controlContext;
};

exports.useCurrentControl = useCurrentControl;