"use strict";

exports.__esModule = true;
exports.AsyncProcessor = void 0;

class AsyncProcessor {
  constructor(forceUpdate) {
    this.forceUpdate = forceUpdate;
    this.error = void 0;
    this.data = void 0;
    this.params = void 0;
    this.status = "";

    this.processing = params => {
      this.params = params;
      this.status = "PROCESSING";
      const {
        forceUpdate
      } = this;
      forceUpdate && forceUpdate();
    };

    this.succeed = data => {
      this.data = data;
      this.status = "SUCCESS";
      const {
        forceUpdate
      } = this;
      forceUpdate && forceUpdate();
    };

    this.failed = error => {
      this.error = error;
      this.status = "FAILURE";
      const {
        forceUpdate
      } = this;
      forceUpdate && forceUpdate();
    };

    this.processing = this.processing.bind(this);
    this.succeed = this.succeed.bind(this);
    this.failed = this.failed.bind(this);
  }

  get triggers() {
    return {
      processing: this.processing,
      succeed: this.succeed,
      failed: this.failed
    };
  }

  retry(callback, params) {
    callback(this.triggers, params || this.params);
  }

}

exports.AsyncProcessor = AsyncProcessor;